//=========================================================================--
// This file is a part of the OpenRISC 1001 VHDL model
// Copyright (C) 2000 Damjan Lampret and OPENCORES.ORG
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// See the file COPYING for the full details of the license.
//
// OPENCORES.ORG is looking for new open source IP cores and developers that
// would like to help in our mission.
//---------------------------------------------------------------------------
// IP core: OpenRISC 1001, a 32-bit RISC processor core from OR1K family.
// Features:
//  * fast exception processing (see architecture manual for details)
//  * optimized for size (without MMU and Caches)
//  * optimized for Xilinx Virtex/Spartan2 FPGAs (over 100MHz in -6 parts)
// Documentation: http://www.opencores.org/cores/or1k/or1001/
//----------------------------------------------------------------------------
// File: regfile.vhd
// Entity: regfile
// Author: Damjan Lampret, lampret@opencores.org
// Description:	
//  Register file.
// Architecture Notes:	
//  uses .
// TO DO:
//  * add other dual-port rams (FPGA/ASIC library details welcome) in memory
//  libraries
//---------------------------------------------------------------------------
// Revision History
// $Log: regfile.vhd,v $
// Revision 1.1.1.1  2000/04/18 13:19:42  lampret
// This implementation is now in the middle of modifications to have one CPU core both for OpenRISC 1000 architecture and MIPS I.
//
// Translation to Verilog by Peter Monta <pmonta@terayon.com>, 22 April 2000

`include "config.h"

module regfile(rfaddrw, rfaddr1, rfaddr2,
  rfdataw, rfdata1, rfdata2, rfenw,
  rst, clk);

  parameter datawidth = 32;

  input [3:0] rfaddrw;
  input [3:0] rfaddr1;
  input [3:0] rfaddr2;
  input [datawidth-1:0] rfdataw;
  output [datawidth-1:0] rfdata1;
  output [datawidth-1:0] rfdata2;
  input rfenw;
  input rst;
  input clk;

  reg [datawidth-1:0] rfdata1;
  reg [datawidth-1:0] rfdata2;

  reg [datawidth-1:0] mem[15:0];

  always @(posedge clk) begin
    if (rfenw) begin
      mem[rfaddrw] <= #1 rfdataw;
      $display("regfile: store reg[%x] <- %x",rfaddrw,rfdataw);
    end
    rfdata1 <= #1 mem[rfaddr1];
    rfdata2 <= #1 mem[rfaddr2];
    $display("regfile: fetch reg[%x] %x",rfaddr1,mem[rfaddr1]);
    $display("regfile: fetch reg[%x] %x",rfaddr2,mem[rfaddr2]);
  end

  integer i;
  initial
    for (i=0; i<16; i=i+1)
      mem[i] = 0;

  always @(posedge clk)
    $display($time,,"rfile: %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x",mem[0],mem[1],mem[2],mem[3],mem[4],mem[5],mem[6],mem[7],mem[8],mem[9],mem[10],mem[11],mem[12],mem[13],mem[14],mem[15]);

endmodule
